% MTODO 1
% TEST 1
%
% La referencia utilizada ha sido:
%
% 	M. Matilla-Garca, M. Ruz
%	A NON-PARAMETRIC INDEPENDENCE TEST USING PERMUTATION ENTROPY
%	Journal of Econometrics 144 (2008), 139-155
%
% Realizamos el test a travs de los pasos que se exponen a continuacin:
%
% PASO 0: Se introducen los parmetros de entrada de la funcin, que
% son la serie temporal, la dimensin de inmersin y el nivel de confianza.
%
% PASO 1: Clculos previos.
% PASO 2: Calculamos H (entropa de permutacin) y el estadstico de
% contarste.
% PASO 3: Realizamos el contraste de hiptesis.

function t1=test_1(serie_temp,m,nivel_confianza)

% PASO 1
% CLCULOS PREVIOS

serie=serie_temp; % serie temporal
T=length(serie); % nmero de elementos de la serie
nivel_significacion=1-nivel_confianza/100; % nivel de significacin

% PASO 2
% CALCULAMOS H (ENTROPA DE PERMUTACIN) Y EL ESTADSTICO DE CONTRASTE

% La serie temporal se divide en varios tramos, cumpliendo:
%
%   1) El primer tramo se construye a partir del primer elemento de la
%   serie.
%   2) Cada tramo tiene el mismo nmero de elementos, y este es igual a
%   la dimensin de inmersin m.
%   3) Cada tramo abarca un conjunto de elementos consecutivos.
%   4) El desplazamiento de un tramo hasta el siguiente es de un slo
%   elemento, de modo que hay un solapamiento de m-1 elementos entre tramos
%   consecutivos.
%   

% A cada tramo se le asocia una permutacin, dependiendo sta del orden de
% los elementos que componen dicho tramo.

% Para calcular la entropa de permutacin H, hay que determinar primero la
% frecuencia absoluta de cada una de las posibles permutaciones.

M_permutaciones=zeros(factorial(m),m);
Fabs_permutacion=zeros(factorial(m),1);

trozo_serie1=serie(1:m);
[trozo_ordenado1,permutacion1]=sort(trozo_serie1);
M_permutaciones(1,:)=permutacion1;
Npermutaciones_diferentes=1;
Fabs_permutacion(1)=1;

for cont1=2:T-m+1
    
    trozo_serie=serie(cont1:cont1+m-1);
    [trozo_ordenado,permutacion]=sort(trozo_serie);
        
    testigo=1;
    
    for cont2=1:Npermutaciones_diferentes
        
        if isequal(M_permutaciones(cont2,:),permutacion)
            
            Fabs_permutacion(cont2)=Fabs_permutacion(cont2)+1;
            testigo=0;
            break            
            
        end
        
    end
    
    if testigo
        
        Npermutaciones_diferentes=Npermutaciones_diferentes+1;
        M_permutaciones(Npermutaciones_diferentes,:)=permutacion;
        Fabs_permutacion(Npermutaciones_diferentes)=1;
    
    end
    
end

% Una vez calculadas las frecuencias absolutas de las permutaciones
% calculamos las frecuencias relativas.

Frel_permutacion=Fabs_permutacion/(T-m+1);

% Con las frecuencias relativas ya se puede calcular la entropa de
% permutacin H.
% La expresin matemtica de la entropa de permutacin es:
%
%       H=-SUM(fr_pi*ln(fr_pi))
%
% donde fr_pi es la frecuencia relativa de cada permutacin. 

H=0; % entropa de permutacin 

for cont3=1:Npermutaciones_diferentes
      
    incremento_H=-Frel_permutacion(cont3)*log(Frel_permutacion(cont3));
    H=H+incremento_H;
      
end

% Calculamos el estadstico de contraste G.
% Su frmula de clculo es:
%
%       G=2*(T-m+1)*(ln(m!)-H)
%
% Vemos que el estadstico de contraste depende del tamao de la serie T,
% de la dimensin de inmersin m y de la entropa de permutacin H.

G=2*(T-m+1)*(log(factorial(m))-H); % estadstico de contraste

% PASO 3
% REALIZAMOS EL CONTRASTE DE HIPTESIS

grados_libertad=factorial(m)-1; % grados de libertad
chi_cuadrado=chi2inv(1-nivel_significacion,grados_libertad); % valor crtico

% Si el estadstico de contraste es menor o igual al valor crtico, no
% rechazamos la hiptesis nula; en este caso la variable t1 toma el valor
% 1.
% En caso contrario se rechaza la hiptesis nula, y la variable t1 toma el
% valor 0.

if G<=chi_cuadrado
    
    t1=1;
    
else
        
    t1=0;
        
end
